@@ -160,11 +160,16 @@ fill_prop_sample (GtkHTMLEditTableProperties *d)
 	/* printf ("html: %s\n", html); */
 }
 
-#define TEMPLATES 4
+#define TEMPLATES 3
 typedef struct {
 	gchar *name;
 	gint offset;
 
+	gint default_width;
+	gboolean default_percent;
+
+	HTMLHAlignType default_align;
+
 	gint default_border;
 	gint default_spacing;
 	gint default_padding;
@@ -182,7 +187,7 @@ typedef struct {
 static TableInsertTemplate table_templates [TEMPLATES] = {
 	{
 		N_("Plain"), 1,
-		1, 2, 1, 3, 3,
+		100, TRUE, HTML_HALIGN_CENTER, 1, 2, 1, 3, 3,
 		"<table border=@border@ cellspacing=@spacing@ cellpadding=@padding@@align@@width@>",
 		"</table>",
 		"<td>",
@@ -190,7 +195,7 @@ static TableInsertTemplate table_templates [TEMPLATES] = {
 	},
 	{
 		N_("Flat gray"), 2,
-		0, 1, 3, 3, 3,
+		100, TRUE, HTML_HALIGN_CENTER, 0, 1, 3, 3, 3,
 		"<table cellspacing=0 cellpadding=@border@ bgcolor=\"#bfbfbf\"@width@@align@><tr><td>"
 		"<table bgcolor=\"#f2f2f2\" cellspacing=@spacing@ cellpadding=@padding@ width=\"100%\">",
 		"</table></td></tr></table>",
@@ -199,7 +204,7 @@ static TableInsertTemplate table_templates [TEMPLATES] = {
 	},
 	{
 		N_("Dark header"), 11,
-		0, 1, 3, 3, 3,
+		100, TRUE, HTML_HALIGN_CENTER, 0, 1, 3, 3, 3,
 		"<table bgcolor=\"#7f7f7f\" cellpadding=3 cellspacing=0>"
 		"<tr><td><font color=\"#ffffff\"><b>Header</td></tr></table>"
 		"<table cellspacing=0 cellpadding=@border@ bgcolor=\"#bfbfbf\"@width@@align@><tr><td>"
@@ -208,17 +213,6 @@ static TableInsertTemplate table_templates [TEMPLATES] = {
 		"<td>",
 		"</td>"
 	},
-	{
-		N_("Note"), 5,
-		1, 1, 3, 1, 1,
-		"<table cellspacing=0 cellpadding=@border@ bgcolor=\"#ccccc0\"@width@@align@><tr><td>"
-		"<table bgcolor=\"#fffff0\" cellpadding=3 cellspacing=0 width=\"100%\">"
-		"<tr><td valign=top><img src=\"file://" ICONDIR "/bulb.png\"></td>"
-		"<td width=\"100%\"><table cellspacing=@spacing@ cellpadding=@padding@ width=\"100%\"",
-		"</table></td></tr></table></td></tr></table>",
-		"<td>",
-		"</td>"
-	}
 };
 
 static gchar *
@@ -510,7 +504,6 @@ changed_template (GtkWidget *w, GtkHTMLEditTableProperties *d)
 
 	set_insert_ui (d);
 
-	FILL;
 	CHANGE;	
 }
 
@@ -526,7 +519,6 @@ table_widget (GtkHTMLEditTableProperties *d)
 	HTMLColor *color;
 	GtkWidget *table_page;
 	GladeXML *xml;
-	gchar     *dir;
 
 	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_page");
 	if (!xml)
@@ -548,11 +540,16 @@ table_widget (GtkHTMLEditTableProperties *d)
 	d->check_bg_pixmap = glade_xml_get_widget (xml, "check_table_bg_pixmap");
 	gtk_signal_connect (GTK_OBJECT (d->check_bg_pixmap), "toggled", set_has_bg_pixmap, d);
 	d->entry_bg_pixmap = glade_xml_get_widget (xml, "entry_table_bg_pixmap");
+
+	d->disable_change = TRUE;
+	/* fix for broken gnome-libs, could be removed once gnome-libs are fixed */
+	gnome_entry_load_history (GNOME_ENTRY (gnome_pixmap_entry_gnome_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))));
+	our_gnome_pixmap_entry_set_last_dir (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap));
+	d->disable_change = FALSE;
+
 	gtk_signal_connect (GTK_OBJECT (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap))),
 			    "changed", GTK_SIGNAL_FUNC (changed_bg_pixmap), d);
-	dir = getcwd (NULL, 0);
-	gnome_pixmap_entry_set_pixmap_subdir (GNOME_PIXMAP_ENTRY (d->entry_bg_pixmap), dir);
-	free (dir);
+
 
 	d->spin_spacing = glade_xml_get_widget (xml, "spin_spacing");
 	gtk_signal_connect (GTK_OBJECT (d->spin_spacing), "changed", changed_spacing, d);
@@ -593,6 +590,20 @@ table_widget (GtkHTMLEditTableProperties *d)
 	return table_page;
 }
 
+static void
+fill_templates (GtkHTMLEditTableProperties *d)
+{
+	GtkWidget *menu;
+	gint i;
+
+	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template));
+
+	for (i = 0; i < TEMPLATES; i ++)
+		gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label (_(table_templates [i].name)));
+	gtk_menu_set_active (GTK_MENU (menu), 0);
+	gtk_container_remove (GTK_CONTAINER (menu), gtk_menu_get_active (GTK_MENU (menu)));
+}
+
 static GtkWidget *
 table_insert_widget (GtkHTMLEditTableProperties *d)
 {
@@ -635,20 +646,7 @@ table_insert_widget (GtkHTMLEditTableProperties *d)
 	d->option_template = glade_xml_get_widget (xml, "option_table_template");
 	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template))), "selection-done",
 			    changed_template, d);
-	{
-		GtkWidget *menu;
-		gint i;
-
-		menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template));
-
-		for (i = 0; i < TEMPLATES; i ++)
-			gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label (_(table_templates [i].name)));
-		//gtk_option_menu_remove_menu (GTK_OPTION_MENU (d->option_template));
-		//gtk_option_menu_set_menu (GTK_OPTION_MENU (d->option_template), menu);
-		gtk_menu_set_active (GTK_MENU (menu), 0);
-		gtk_container_remove (GTK_CONTAINER (menu), gtk_menu_get_active (GTK_MENU (menu)));
-	}
-
+	fill_templates (d);
 	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), FALSE, FALSE, 0);
 
 	gtk_widget_show_all (table_page);
@@ -679,9 +677,6 @@ set_ui (GtkHTMLEditTableProperties *d)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_width),  d->width);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_width), d->width_percent ? 1 : 0);
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_width), d->width_percent ? 1 : 0);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_width), d->width_percent ? 1 : 0);
-
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_cols),  d->cols);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_rows),  d->rows);
 
